"use strict";

window.onload = init;

let gl; // kontekst WebGL

let vPosition; // lokalizacja atrybutu - wspolrzedne wierzcholka
let vColor; // lokalizacja atrybutu - kolor wierzcholka
let program; // obiekt programu cieniowania

// dane o trojkacie
let triangle =
{
    "positionVBO" : null, // VBO zawierajace wspolrzedne
    "colorVBO" : null, // VBO zawierajace kolory
    "noVertices" : 0 // liczba wierzcholkow do wyrenderowania
};

// dane o kwadracie
let square =
{
    "interleavedVBO" : null, // VBO zawierajace przeplatane wspolrzedne i kolory
    "stride" : 0, // wielkosc jednego elementu (wspolrzedne, kolor)
    "colorStart" : 0, // od jakiego miejsca w elemencie zaczyna sie kolor
    "noVertices" : 0 // liczba wierzcholkow do wyrenderowania
};

/*
* funkcja inicjuje WebGL oraz ustawia poczatkowe wartosci pewnych ustawien WebGL
*/
function init()
{
    initGL();

    setupShaders();
    setupBuffers();

    render();
}

/*
* funkcja pobiera kontekst WebGL oraz inicjuje pewne ustawienia WebGL
*/
function initGL()
{
    let canvas = document.getElementById( "gl-canvas" );

    gl = WebGLUtils.setupWebGL( canvas );
    if( !gl )
    {
        alert( "WebGL jest niedostępny!" );
        throw new Error( "WebGL isn't available!" );
    }

    gl.viewport( 0, 0, canvas.width, canvas.height );
    gl.clearColor( 0.7, 0.7, 0.7, 1.0 );
}

/*
* funkcja wczytuje shadery oraz pobiera lokalizacje atrybutow i zmiennych jednorodnych
*/
function setupShaders()
{
    program = createProgram( gl, "vertex-shader", "fragment-shader" );

    if( !program )
        throw new Error( "Shader program creation failed!" );

    vPosition = gl.getAttribLocation( program, "vPosition" );
    vColor = gl.getAttribLocation( program, "vColor" );
}

/*
* funkcja renderuje scene
*/
function render()
{
    gl.clear( gl.COLOR_BUFFER_BIT );

    gl.useProgram( program ); // wlaczamy program cieniowania

    gl.enableVertexAttribArray( vPosition );
    gl.enableVertexAttribArray( vColor );

    // trojkat
    gl.bindBuffer( gl.ARRAY_BUFFER, triangle.positionVBO ); // dowiazujemy obiekt VBO
    gl.vertexAttribPointer( vPosition, 4, gl.FLOAT, false, 0, 0 ); // wiazemy dane z VBO z tablica dla atrybutu
    gl.bindBuffer( gl.ARRAY_BUFFER, triangle.colorVBO );
    gl.vertexAttribPointer( vColor, 3, gl.FLOAT, false, 0, 0 );

    gl.drawArrays( gl.TRIANGLES, 0, triangle.noVertices ); // renderujemy

    // kwadrat
    gl.bindBuffer( gl.ARRAY_BUFFER, square.interleavedVBO );
    gl.vertexAttribPointer( vPosition, 4, gl.FLOAT, false, square.stride, 0 );
    gl.vertexAttribPointer( vColor, 3, gl.FLOAT, false, square.stride, square.colorStart );

    gl.drawArrays( gl.TRIANGLE_STRIP, 0, square.noVertices );
}

/*
* funkcja tworzy obiekty VBO z modelami do wyrenderowania
*/
function setupBuffers()
{
    // trojkat
    let positions =
    [
        0.1, 0.0, 0.0, 1.0,
		0.9, -0.5, 0.0, 1.0,
		0.9, 0.5, 0.0, 1.0
    ];
    let colors =
    [
        1.0, 0.0, 0.0,
        0.0, 1.0, 0.0,
        0.0, 0.0, 1.0
    ];

    triangle.positionVBO = gl.createBuffer(); // tworzymy id obiektu VBO
    gl.bindBuffer( gl.ARRAY_BUFFER, triangle.positionVBO ); // dowiazujemy obiekt VBO
    gl.bufferData( gl.ARRAY_BUFFER, new Float32Array( positions ), gl.STATIC_DRAW ); // tworzymy i ladujemy dane do VBO

    triangle.colorVBO = gl.createBuffer();
    gl.bindBuffer( gl.ARRAY_BUFFER, triangle.colorVBO );
    gl.bufferData( gl.ARRAY_BUFFER, new Float32Array( colors ), gl.STATIC_DRAW );

    triangle.noVertices = 3;

    // kwadrat (x, y, z, w, r, g, b)
    let positionsAndColors =
	[
		-0.9, 0.4, 0.0, 1.0, 0.0, 0.0, 1.0,
		-0.9, -0.4, 0.0, 1.0, 0.0, 1.0, 0.0,
		-0.1, 0.4, 0.0, 1.0, 0.0, 1.0, 0.0,
		-0.1, -0.4, 0.0, 1.0, 0.0, 0.0, 1.0
	];

    square.interleavedVBO = gl.createBuffer();

    gl.bindBuffer( gl.ARRAY_BUFFER, square.interleavedVBO );
    gl.bufferData( gl.ARRAY_BUFFER, new Float32Array( positionsAndColors ), gl.STATIC_DRAW );

    square.stride = 7 * Float32Array.BYTES_PER_ELEMENT; // 7 = 4 wspolrzedne + 3 kanaly koloru
    square.colorStart = 4 * Float32Array.BYTES_PER_ELEMENT;
    square.noVertices = 4;
}
